/*
 * @Copyright:  杭州藏茗山科技有限公司  All rights reserved.Notice 仅限于授权后使用，禁止非授权传阅以及私自用于商业目的。
 */

package com.jeecms.reinsurance.service.impl;

import com.jeecms.common.constants.WebConstants;
import com.jeecms.common.exception.GlobalRuntimeException;
import com.jeecms.common.exception.SystemExceptionEnum;
import com.jeecms.common.util.StrUtils;
import com.jeecms.common.util.XssUtil;
import com.jeecms.common.web.util.RequestUtils;
import com.jeecms.common.web.util.UrlUtil;
import com.jeecms.protection.service.GradeProtectionService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Base64Utils;

import javax.servlet.http.HttpServletRequest;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

import static com.jeecms.common.constants.TplConstants.RES_PATH;


/**
 * 等级保护service实现类
 * @author: chenming
 * @date: 2020/7/30 15:23
 */
@Service
@Transactional(rollbackFor = Exception.class)
@ConditionalOnClass(name =  "com.jeecms.reinsurance.service.impl.CmsReinsuranceConfigServiceImpl")
public class GradeProtectionServiceImpl implements GradeProtectionService {

    static Logger logger = LoggerFactory.getLogger(GradeProtectionServiceImpl.class);

    @Value("${security.openParamBase64}")
    private String openParamBase64;

    @Override
    public boolean checkGradeProtection()  {
        return WebConstants.TRUE.equals(openParamBase64);
    }

    @Override
    public List<String> decryptStrList(List<String> requestStrList) {
        if(!checkGradeProtection()){
            return requestStrList;
        }
        List<String> stringList = new ArrayList<>();
        for (String requestStr:requestStrList) {
            stringList.add(this.decryptStr(requestStr));
        }
        return stringList;
    }

    @Override
    public String decryptStr(String requestStr) {
        if(!checkGradeProtection()){
            return requestStr;
        }
        if (StringUtils.isNotBlank(requestStr)) {
            //转的报错认为是没有base64 等于原值
            String str = requestStr;
            //反解之后还是base64 则认定前端多次base64 则不断迭代反解
            //这里前端文件管理里面有部分接口请求没有base64直接返回
            if (str.startsWith(RES_PATH)) {
                return str;
            }
            try {
                ///**尝试decode两次判断是否有特殊字符*/
                str = new String(Base64.getDecoder().decode(requestStr), StandardCharsets.UTF_8);
                //URLDecoder decode会吃掉 js里面的 +
//                str = URLDecoder.decode(str, StandardCharsets.UTF_8.name());
                str = str.replaceAll("%(?![0-9a-fA-F]{2})", "%25");
                //JS + 不能替换
                //str = str.replaceAll("\\+", "%2B");
//                str = new String(Base64Utils.decodeFromUrlSafeString(str), StandardCharsets.UTF_8);
                //                /**URLDecoder: Incomplete trailing escape (%) pattern*/
            } catch (Exception e) {
                logger.error("base64解密失败->" + e.getMessage());
                //之前判断是否base64不准确，不断反转64 报错了就是正确的结果了
                return str;
            }
            //jeegov 正常字符反解 乱码了
            if (!StrUtils.isBase64(str)) {
                return str;
            }
            /**base64反解密发现有注入字符串提示*/
            /**模板 资源是允许包含特殊字符的，本身就是包含js的所以跳过检查*/
            /**这里是需要反转base64的功能区域，如果是内容管理的则提前返回不校验xss*/
            HttpServletRequest request = RequestUtils.getHttpServletRequest();
            String uri = request.getRequestURI();
            String ctx = request.getContextPath();
            if (StringUtils.isNoneBlank(ctx)) {
                uri = uri.substring(ctx.length());
            }
            if (uri.startsWith(WebConstants.CONTENT_URL)) {
                return str;
            }
            if (!UrlUtil.isContainerSpecialCharUrl(request) && XssUtil.containXss(str)) {
                logger.info(
                        "======访问地址发现非法字符，已拦截======其非法地址为：" + request.getRequestURL().toString());
                throw new GlobalRuntimeException(SystemExceptionEnum.ILLEGAL_PARAM);
            }
            //之前判断是否base64不准确，不断反转64 报错了就是正确的结果了
            return decryptStr(str);
        }
        return requestStr;
    }
}
